print("import: 'databricks_cli'")
import databricks_cli

print("import: 'databricks_cli.cluster_policies'")
import databricks_cli.cluster_policies

print("import: 'databricks_cli.clusters'")
import databricks_cli.clusters

print("import: 'databricks_cli.configure'")
import databricks_cli.configure

print("import: 'databricks_cli.dbfs'")
import databricks_cli.dbfs

print("import: 'databricks_cli.groups'")
import databricks_cli.groups

print("import: 'databricks_cli.instance_pools'")
import databricks_cli.instance_pools

print("import: 'databricks_cli.jobs'")
import databricks_cli.jobs

print("import: 'databricks_cli.libraries'")
import databricks_cli.libraries

print("import: 'databricks_cli.oauth'")
import databricks_cli.oauth

print("import: 'databricks_cli.pipelines'")
import databricks_cli.pipelines

print("import: 'databricks_cli.repos'")
import databricks_cli.repos

print("import: 'databricks_cli.runs'")
import databricks_cli.runs

print("import: 'databricks_cli.sdk'")
import databricks_cli.sdk

print("import: 'databricks_cli.secrets'")
import databricks_cli.secrets

print("import: 'databricks_cli.stack'")
import databricks_cli.stack

print("import: 'databricks_cli.tokens'")
import databricks_cli.tokens

print("import: 'databricks_cli.unity_catalog'")
import databricks_cli.unity_catalog

print("import: 'databricks_cli.workspace'")
import databricks_cli.workspace

